#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <rsbac/types.h>
#include <rsbac/aci_data_structures.h>
#include <rsbac/getname.h>
#include <rsbac/syscalls.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>

int verbose=0;
int recurse=0;
union rsbac_attribute_value_t value;
enum rsbac_target_t target;
enum rsbac_attribute_t attr;
char * progname;

int process(char * name)
  {
    int res = 0;
    char tmp1[120];
    enum rsbac_target_t type;
    char type_n[5] = "";
    struct stat buf;

    if (lstat(name,&buf))
      {
        fprintf(stderr, "stat for file %s returned error: %s\n",
                name,
                strerror(errno));
        return(1);
      }
    if(S_ISDIR(buf.st_mode))
      {
        type = T_DIR;
        strcpy(type_n,"DIR");
        if(target == T_FILE)
          {
            if(verbose)
              printf("%s is no file, skipped\n",
                     name);
            goto do_recurse;
          }
      }
    else
      if(   (S_ISREG(buf.st_mode))
         || (S_ISBLK(buf.st_mode))
         || (S_ISCHR(buf.st_mode)) )
        {
          if(target == T_DIR)
            {
              if(verbose)
                printf("%s is no dir, skipped\n",
                       name);
              return(-1);
            }
          type = T_FILE;
          strcpy(type_n,"FILE");
        }
      else
        {
          if(verbose)
            printf("%s is neither file nor dir, skipped\n",
                   name);
          return(-1);
        }
    if(verbose)
      printf("Processing %s '%s'\n",
             type_n,
             name);
    if(target == T_FD)
      res = rsbac_remove_target_n(type, name);
    else
      res = rsbac_remove_target_n(target, name);
    if(res)
      {
        get_error_name(tmp1,res);
        fprintf(stderr, "%s: error: %s\n", name, tmp1);
      }

do_recurse:
    if (   (type == T_DIR)
        && recurse)
      {
        DIR * dir_stream_p;
        struct dirent * dirent_p;
        char name2[512];

        if(S_ISLNK(buf.st_mode))
          return(0);
        if(!(dir_stream_p = opendir(name)))
          {
            fprintf(stderr, "opendir for dir %s returned error: %s\n",
                   name,
                   strerror(errno));
            return(-2);
          }
        while((dirent_p = readdir(dir_stream_p)))
          {
            if(   (strcmp(".",dirent_p->d_name))
               && (strcmp("..",dirent_p->d_name)) )
              {
                strcpy(name2,name);
                strcat(name2,"/");
                strcat(name2,dirent_p->d_name);
                process(name2);
              }
          }
        closedir(dir_stream_p);
      }
    return(0);
  }

int main(int argc, char ** argv)
{
  int res = 0;
  int i;
  
  progname = argv[0];
  if ((argc > 1) && (!strcmp("-v",argv[1])) )
    {
      verbose=1;
      argv++;
      argc--;
    }
  if ((argc > 1) && (!strcmp("-r",argv[1])) )
    {
      recurse=1;
      argv++;
      argc--;
    }
  if (argc > 2)
    {
      if(verbose)
        printf("%s: %i targets\n\n", progname, argc - 4);
      target = get_target_nr(argv[1]);
      if(   (target != T_DIR)
         && (target != T_FILE)
         && (target != T_DEV)
         && (target != T_FD)
        )
        {
          fprintf(stderr, "%s: Invalid target type %s\n", progname, argv[1]);
          exit(1);
        }
      for (i=1;i < (argc-1);i++)
        {
          process(argv[i+1]);
        }
    }
  else
    {
      printf("%s (RSBAC %s)\n***\n", progname, VERSION);
      printf("Use: %s [-v] [-r] target-type file/dirname(s)\n", progname);  
      printf("- -v = verbose, -r = recurse into subdirs,\n");
      printf("- target-type = FILE, DIR, DEV or FD\n");
      printf("  (FD: let %s decide between FILE and DIR, no DEV),\n", progname);
    }
  return (res);
}

